@@ -1,11 +1,11 @@
 import dataclasses
-from typing import Any, Dict, Generic, List, Optional, Type, TypeVar, Union
+from typing import Any, Dict, Generic, List, Optional, TypeVar, Union
 
 from graphql import OperationDefinitionNode
 from graphql.language import FieldNode
 from graphql.pyutils.path import Path
 
-from strawberry.union import StrawberryUnion
+from strawberry.type import StrawberryType
 
 
 ContextType = TypeVar("ContextType")
@@ -19,8 +19,8 @@ class Info(Generic[ContextType, RootValueType]):
     context: ContextType
     root_value: RootValueType
     variable_values: Dict[str, Any]
-    # TODO: update to StrawberryType once it's implemented
-    return_type: Optional[Union[Type, StrawberryUnion]]
+    # TODO: merge type with StrawberryType when StrawberryObject is implemented
+    return_type: Optional[Union[type, StrawberryType]]
     # TODO: create an abstraction on these fields
     operation: OperationDefinitionNode
     path: Path
